@@ -4,7 +4,7 @@ from ._metadata import __version__, within_flit
 
 if not within_flit():
     del within_flit
-    from ._core.anndata import AnnData, ImplicitModificationWarning
+    from ._core.anndata import AnnData
     from ._core.merge import concat
     from ._core.raw import Raw
     from ._io import (
@@ -18,6 +18,7 @@ if not within_flit():
         read_mtx,
         read_zarr,
     )
+    from ._warnings import OldFormatWarning, WriteWarning, ImplicitModificationWarning
 
     # backwards compat / shortcut for default format
     from ._io import read_h5ad as read
