@@ -3,6 +3,7 @@ from functools import reduce
 from h5py import Dataset
 import numpy as np
 import pandas as pd
+import warnings
 
 from typing import Dict, Union, Optional, Sequence, Callable
 
@@ -13,7 +14,6 @@ from ..._core.views import _resolve_idx
 from ..._core.merge import concat_arrays, inner_concat_aligned_mapping
 from ..._core.sparse_dataset import SparseDataset
 from ..._core.aligned_mapping import AxisArrays
-from ...logging import anndata_logger as logger
 
 ATTRS = ["obs", "obsm", "layers"]
 
@@ -466,6 +466,16 @@ class AnnCollectionView(_ConcatViewMixin, _IterateViewMixin):
         """Shape of the lazily concatenated subset of the data matrix."""
         return len(self.obs_names), len(self.var_names)
 
+    @property
+    def n_obs(self):
+        """Number of observations."""
+        return self.shape[0]
+
+    @property
+    def n_vars(self):
+        """Number of variables/features."""
+        return self.shape[1]
+
     @property
     def convert(self):
         """On the fly converters for keys of attributes and data matrix.
@@ -715,7 +725,7 @@ class AnnCollection(_ConcatViewMixin, _IterateViewMixin):
         self.obs_names = pd.Index(concat_indices)
 
         if not self.obs_names.is_unique:
-            logger.info("Observation names are not unique.")
+            warnings.warn("Observation names are not unique.", UserWarning)
 
         view_attrs = ATTRS.copy()
 
